@@ -13,6 +13,8 @@ import textwrap
 
 import toml
 
+from .exceptions import NotParseableError
+
 
 class Config:
     """Class for accessing and loading pccc configuration options.
@@ -31,11 +33,20 @@ class Config:
     body_length : int
        Maximum body line length; default is 72.
     repair : boolean
-      Repair commit, implying spell check and rewrap; default is ``False``.
+        Repair commit, implying spell check and rewrap; default is
+        ``False``.
     rewrap : boolean
-      Rewrap body; default is ``False``.
+        Rewrap body; default is ``False``.
     spell_check : boolean
-      Spell check header and body; default is ``False``.
+        Spell check header and body; default is ``False``.
+    ignore_generated_commits : boolean
+        Ignore generated commits which match ``generated_commits``
+        regular expressions; default is ``False``.
+    generated_commits : [string]
+        List of generated commits, as Python regular expressions;
+        default is ``[]``.  For TOML files, it's probably best to use
+        the multiline single quote strings to reduce escaping
+        problems.
     types : [string]
         List of header types; default is ``['feat', 'fix']``.
     scopes : [string]
@@ -55,6 +66,8 @@ class Config:
         repair=False,
         rewrap=False,
         spell_check=False,
+        ignore_generated_commits=False,
+        generated_commits=[],
         types=["feat", "fix"],
         scopes=[],
         footers=[],
@@ -69,6 +82,7 @@ class Config:
         object
             A Config() object.
         """
+        # Configuration attributes.
         self.commit = commit
         self.config_file = config_file
         self.header_length = header_length
@@ -76,16 +90,28 @@ class Config:
         self.repair = repair
         self.rewrap = rewrap
         self.spell_check = spell_check
+        self.ignore_generated_commits = ignore_generated_commits
+        self.generated_commits = generated_commits
         self.types = types
         self.scopes = scopes
         self.footers = footers
         self.required_footers = required_footers
 
+        # Other attributes.
+        self.format = "TOML"
+
     def __str__(self):
         """Stringify a ``Config()`` object.
 
         String representation of a ``Config()`` object, as the
-        ``[tool.pccc]`` section of a pyproject.toml file.
+        ``[tool.pccc]`` section of a ``pyproject.toml`` file or as the
+        ``pccc`` entry of a JSON file such as ``package.json``.
+
+        Parameters
+        ----------
+        fmt : string
+           The output format.  Default is ``"TOML"``.  Currently
+           supported values are ``"TOML"`` and ``"JSON"``.
 
         Returns
         -------
@@ -93,66 +119,12 @@ class Config:
             The current configuration, as the [tool.pccc] section of a
             ``pyproject.toml`` file.
         """
-        rs = "[tool.pccc]\n"
-        rs += "\n"
-        rs += f"header_length = {self.header_length}\n"
-        rs += f"body_length = {self.body_length}\n"
-
-        if self.repair:
-            rs += "repair = true\n"
+        if self.format == "JSON":
+            # JSON, if requested.
+            rs = json.dumps({"pccc": self.config_as_dict()}, indent=2)
         else:
-            rs += "repair = false\n"
-
-        if self.rewrap:
-            rs += "rewrap = true\n"
-        else:
-            rs += "rewrap = false\n"
-
-        if self.spell_check:
-            rs += "spell_check = true\n"
-        else:
-            rs += "spell_check = false\n"
-
-        rs += "\n"
-
-        # rs += "types = [\n" + "\n".join(map(lambda item: f'  "{item}",', self.types))
-        rs += "types = [\n" + ",\n".join(map(lambda item: f'  "{item}"', self.types))
-
-        if len(self.types):
-            rs += "\n]\n\n"
-        else:
-            rs += "]\n\n"
-
-        rs += "scopes = [\n" + ",\n".join(map(lambda item: f'  "{item}"', self.scopes))
-
-        if len(self.scopes):
-            rs += "\n]\n\n"
-        else:
-            rs += "]\n\n"
-
-        # rs += "footers = [\n" + "\n".join(
-        #     map(lambda item: f'  "{item}",', self.footers)
-        # )
-        rs += "footers = [\n" + ",\n".join(
-            map(lambda item: f'  "{item}"', self.footers)
-        )
-
-        if len(self.footers):
-            rs += "\n]\n\n"
-        else:
-            rs += "]\n\n"
-
-        # rs += "required_footers = [\n" + "\n".join(
-        #     map(lambda item: f'  "{item}",', self.required_footers)
-        # )
-        rs += "required_footers = [\n" + ",\n".join(
-            map(lambda item: f'  "{item}"', self.required_footers)
-        )
-
-        if len(self.required_footers):
-            rs += "\n]\n"
-        else:
-            rs += "]\n"
+            # TOML, by default.
+            rs = toml.dumps({"tool": {"pccc": self.config_as_dict()}})
 
         return rs
 
@@ -166,12 +138,55 @@ class Config:
             f"repair={self.repair}, "
             f"rewrap={self.rewrap}, "
             f"spell_check={self.spell_check}, "
+            f"ignore_generated_commits={self.ignore_generated_commits}, "
+            f"generated_commits={self.generated_commits}, "
             f"types={self.types}, "
             f"scopes={self.scopes}, "
             f"footers={self.footers}, "
-            f"required_footers={self.required_footers})"
+            f"required_footers={self.required_footers}, "
+            f'format="{self.format}")'
         )
 
+    def set_format(self, format):
+        """Set the output format.
+
+        Set the output format of ``Config().__str__()`` to either
+        ``"TOML"`` or ``"JSON"``.
+        """
+        if format == "JSON":
+            self.format = "JSON"
+        else:
+            self.format = "TOML"
+
+        return
+
+    def config_as_dict(self):
+        """Convert a ``Config()`` object to a ``dict``.
+
+        Convert a ``Config()`` object's configuration parameters to a
+        ``dict`` for using in dumping TOML and JSON data.
+
+        Returns
+        -------
+        dict
+            A ``dict`` containing key-value pairs of the configuration
+            parameters and their values.
+        """
+        return {
+            "commit": self.commit,
+            "header_length": self.header_length,
+            "body_length": self.body_length,
+            "repair": self.repair,
+            "rewrap": self.rewrap,
+            "spell_check": self.spell_check,
+            "ignore_generated_commits": self.ignore_generated_commits,
+            "generated_commits": self.generated_commits,
+            "types": self.types,
+            "scopes": self.scopes,
+            "footers": self.footers,
+            "required_footers": self.required_footers,
+        }
+
     def update(self, *args, **kwargs):
         """Update a configuration.
 
@@ -294,6 +309,7 @@ def _load_file(filename="./pyproject.toml"):
     """
     options = {}
     jsonRE = re.compile(r"^.*\.json$", re.IGNORECASE)
+    tomlRE = re.compile(r"^.*\.toml$", re.IGNORECASE)
 
     if os.path.abspath(filename) == os.path.abspath("./pyproject.toml"):
         try:
@@ -302,13 +318,8 @@ def _load_file(filename="./pyproject.toml"):
         except toml.TomlDecodeError:
             raise
         except FileNotFoundError:
-            try:
-                # Then try ``./package.json``.
-                options = _load_json_file("./package.json")
-            except json.JSONDecodeError:
-                raise
-            except FileNotFoundError:
-                raise
+            # No file, bail.
+            raise
     elif jsonRE.match(filename):
         try:
             # Well, if JSON is supplied, use it.
@@ -317,12 +328,23 @@ def _load_file(filename="./pyproject.toml"):
             raise
         except FileNotFoundError:
             raise
+    elif tomlRE.match(filename):
+        try:
+            options = _load_toml_file(filename)
+        except toml.TomlDecodeError:
+            raise
+        except FileNotFoundError:
+            raise
     else:
         try:
             # Last chance, parse filename as TOML.
             options = _load_toml_file(filename)
         except toml.TomlDecodeError:
-            raise
+            # Okay, one last try.  Maybe it was JSON.
+            try:
+                options = _load_json_file(filename)
+            except json.JSONDecodeError:
+                raise NotParseableError(filename)
         except FileNotFoundError:
             raise
 
@@ -377,12 +399,15 @@ def _load_json_file(filename="./package.json"):
         "header_length": None,
         "body_length": None,
         "spell_check": None,
+        "ignore_generated_commits": None,
         "rewrap": None,
         "repair": None,
         "types": None,
         "scopes": None,
         "footers": None,
         "required_footers": None,
+        "ignore_generated_commits": None,
+        "generated_commits": None,
     }
 
     for (k, v) in config["pccc"].items():
@@ -446,6 +471,8 @@ def _load_toml_file(filename="./pyproject.toml"):
         "scopes": None,
         "footers": None,
         "required_footers": None,
+        "ignore_generated_commits": None,
+        "generated_commits": None,
     }
 
     for (k, v) in config["tool"]["pccc"].items():
@@ -536,6 +563,26 @@ to redistribute it under certain conditions; type ``pccc
         help="Do not spell check the commit.  Default is no spell checking.",
     )
 
+    ignore_group = parser.add_mutually_exclusive_group()
+    ignore_group.add_argument(
+        "-i",
+        "--ignore-generated-commits",
+        dest="ignore_generate_commits",
+        default=None,
+        action="store_true",
+        help="Ignore generated commits that match the patterns in"
+        " ``generated_commits``.  Default is to check every commit.",
+    )
+    ignore_group.add_argument(
+        "-I",
+        "--no-ignore-generated-commits",
+        dest="ignore_generated_commits",
+        default=None,
+        action="store_false",
+        help="Do not ignore generated commits that match the patterns in"
+        " ``generated_commits``.  Default is to check every commit.",
+    )
+
     rewrap_group = parser.add_mutually_exclusive_group()
     rewrap_group.add_argument(
         "-w",
@@ -586,6 +633,18 @@ to redistribute it under certain conditions; type ``pccc
         " of header.  Default is `['fix', 'feat']`.",
     )
 
+    parser.add_argument(
+        # "a" is for automatic.
+        "-a",
+        "--generated-commits",
+        dest="generated_commits",
+        default=None,
+        type=_field_list_handler,
+        help="List (comma delimited) of Python regular expressions that"
+        " match generated commits that should be ignored.  Mind the shell"
+        " escaping.  Default is ``[]``.",
+    )
+
     parser.add_argument(
         "-s",
         "--scopes",
